@@ -1,7 +1,7 @@
 /* deliver.c -- deliver shell - just calls lmtpd
  * Tim Martin
  *
- * Copyright (c) 2000 Carnegie Mellon University.  All rights reserved.
+ * Copyright (c) 1998-2003 Carnegie Mellon University.  All rights reserved.
  *
  * Redistribution and use in source and binary forms, with or without
  * modification, are permitted provided that the following conditions
@@ -41,7 +41,7 @@
  *
  */
 
-/* $Id: deliver.c,v 1.165 2002/08/02 16:54:48 ken3 Exp $ */
+/* $Id: deliver.c,v 1.174 2004/05/22 03:45:48 rjs3 Exp $ */
 
 #include <config.h>
 
@@ -56,7 +56,6 @@
 #include <fcntl.h>
 #include <sys/stat.h>
 #include <syslog.h>
-#include <com_err.h>
 #include <errno.h>
 #include <pwd.h>
 #include <sys/types.h>
@@ -67,7 +66,7 @@
 #include <arpa/inet.h>
 #include <sys/un.h>
 
-#include "imapconf.h"
+#include "global.h"
 #include "exitcodes.h"
 #include "imap_err.h"
 #include "xmalloc.h"
@@ -75,6 +74,9 @@
 #include "prot.h"
 #include "version.h"
 
+/* config.c stuff */
+const int config_need_data = CONFIG_NEED_PARTITION_DATA;
+
 extern int optind;
 extern char *optarg;
 
@@ -94,8 +96,7 @@ static int deliver_msg(char *return_path, char *authuser, int ignorequota,
 		       char **users, int numusers, char *mailbox);
 static int init_net(const char *sockaddr);
 
-static void
-usage()
+static void usage()
 {
     fprintf(stderr, 
 	    "421-4.3.0 usage: deliver [-C <alt_config> ] [-m mailbox]"
@@ -106,8 +107,14 @@ usage()
 
 void fatal(const char* s, int code)
 {
+    static int recurse_code = 0;
+    
+    if(recurse_code) exit(code);
+    else recurse_code = 0;
+    
     prot_printf(deliver_out,"421 4.3.0 deliver: %s\r\n", s);
     prot_flush(deliver_out);
+    cyrus_done();
     exit(code);
 }
 
@@ -183,6 +190,7 @@ void pipe_through(int lmtp_in, int lmtp_out, int local_in, int local_out)
 
 int main(int argc, char **argv)
 {
+    int r;
     int opt;
     int lmtpflag = 0;
     int ignorequota = 0;
@@ -215,6 +223,7 @@ int main(int argc, char **argv)
 	    if (mailboxname) {
 		fprintf(stderr, "deliver: multiple -m options\n");
 		usage();
+		/* NOTREACHED */
 	    }
 	    if (*optarg) mailboxname = optarg;
 	    break;
@@ -223,6 +232,7 @@ int main(int argc, char **argv)
 	    if (authuser) {
 		fprintf(stderr, "deliver: multiple -a options\n");
 		usage();
+		/* NOTREACHED */
 	    }
 	    authuser = optarg;
 	    break;
@@ -251,6 +261,7 @@ int main(int argc, char **argv)
 
 	default:
 	    usage();
+	    /* NOTREACHED */
 	}
     }
 
@@ -259,12 +270,12 @@ int main(int argc, char **argv)
     prot_setflushonread(deliver_in, deliver_out);
     prot_settimeout(deliver_in, 300);
 
-    config_init(alt_config, "deliver");
+    cyrus_init(alt_config, "deliver", CYRUSINIT_NODB);
 
-    sockaddr = config_getstring("lmtpsocket", NULL);
+    sockaddr = config_getstring(IMAPOPT_LMTPSOCKET);
     if (!sockaddr) {	
-	strcpy(buf, config_dir);
-	strcat(buf, "/socket/lmtp");
+	strlcpy(buf, config_dir, sizeof(buf));
+	strlcat(buf, "/socket/lmtp", sizeof(buf));
 	sockaddr = buf;
     }
 
@@ -281,8 +292,12 @@ int main(int argc, char **argv)
     }
 
     /* deliver to users or global mailbox */
-    return deliver_msg(return_path,authuser, ignorequota,
-		       argv+optind, argc - optind, mailboxname);
+    r = deliver_msg(return_path,authuser, ignorequota,
+		    argv+optind, argc - optind, mailboxname);
+
+    cyrus_done();
+
+    return r;
 }
 
 void just_exit(const char *msg)
@@ -304,7 +319,7 @@ static int init_net(const char *unixpath)
   }
 
   addr.sun_family = AF_UNIX;
-  strcpy(addr.sun_path, unixpath);
+  strlcpy(addr.sun_path, unixpath, sizeof(addr.sun_path));
 
   if (connect(lmtpdsock, (struct sockaddr *) &addr, 
 	      sizeof(addr.sun_family) + strlen(addr.sun_path) + 1) < 0) {
@@ -318,7 +333,7 @@ static int deliver_msg(char *return_path, char *authuser, int ignorequota,
 		       char **users, int numusers, char *mailbox)
 {
     int r;
-    struct lmtp_conn *conn;
+    struct backend *conn;
     struct lmtp_txn *txn = LMTP_TXN_ALLOC(numusers ? numusers : 1);
     int j;
     int ml = 0;
@@ -329,8 +344,8 @@ static int deliver_msg(char *return_path, char *authuser, int ignorequota,
     }
 
     /* connect */
-    r = lmtp_connect(sockaddr, NULL, &conn);
-    if (r) {
+    conn = backend_connect(NULL, sockaddr, &protocol[PROTOCOL_LMTP], "", NULL);
+    if (!conn) {
 	just_exit("couldn't connect to lmtpd");
     }
 
@@ -339,22 +354,32 @@ static int deliver_msg(char *return_path, char *authuser, int ignorequota,
     txn->auth = authuser;
     txn->data = deliver_in;
     txn->isdotstuffed = 0;
+    txn->tempfail_unknown_mailbox = 0;
     txn->rcpt_num = numusers ? numusers : 1;
     if (mailbox) ml = strlen(mailbox);
     if (numusers == 0) {
 	/* just deliver to mailbox 'mailbox' */
-	const char *BB = config_getstring("postuser", "");
-	txn->rcpt[0].addr = (char *) xmalloc(ml + strlen(BB) + 2); /* leaks! */
+	const char *BB = config_getstring(IMAPOPT_POSTUSER);
+	txn->rcpt[0].addr = (char *) xmalloc(ml + strlen(BB) + 2); /* xxx leaks! */
 	sprintf(txn->rcpt[0].addr, "%s+%s", BB, mailbox);
 	txn->rcpt[0].ignorequota = ignorequota;
     } else {
 	/* setup each recipient */
 	for (j = 0; j < numusers; j++) {
 	    if (mailbox) {
+		size_t ulen;
+
 		/* we let it leak ! */
 		txn->rcpt[j].addr = 
 		    (char *) xmalloc(strlen(users[j]) + ml + 2);
-		sprintf(txn->rcpt[j].addr, "%s+%s", users[j], mailbox);
+
+		/* find the length of the userid minus the domain */
+		ulen = strcspn(users[j], "@");
+		sprintf(txn->rcpt[j].addr, "%.*s+%s", ulen, users[j], mailbox);
+
+		/* add the domain if we have one */
+		if (ulen < strlen(users[j]))
+		    strcat(txn->rcpt[j].addr, users[j]+ulen);
 	    } else {
 		txn->rcpt[j].addr = users[j];
 	    }
@@ -366,7 +391,7 @@ static int deliver_msg(char *return_path, char *authuser, int ignorequota,
     r = lmtp_runtxn(conn, txn);
 
     /* disconnect */
-    lmtp_disconnect(conn);
+    backend_disconnect(conn, &protocol[PROTOCOL_LMTP]);
 
     /* examine txn for error state */
     r = 0;
