@@ -1,6 +1,6 @@
 /* mupdate-client.c -- cyrus murder database clients
  *
- * $Id: mupdate-client.c,v 1.45 2004/07/07 19:49:05 rjs3 Exp $
+ * $Id: mupdate-client.c,v 1.47 2005/11/23 13:40:09 murch Exp $
  * Copyright (c) 1998-2003 Carnegie Mellon University.  All rights reserved.
  *
  * Redistribution and use in source and binary forms, with or without
@@ -73,7 +73,6 @@
 #include "imparse.h"
 #include "iptostring.h"
 #include "mupdate.h"
-#include "mupdate_err.h"
 #include "prot.h"
 #include "protocol.h"
 #include "xmalloc.h"
@@ -211,6 +210,7 @@ int mupdate_connect(const char *server, const char *port,
     saslresult=sasl_setprop(h->saslconn, SASL_SEC_PROPS, secprops);
     if(saslresult != SASL_OK) goto noconn;
     free(secprops);
+    secprops = NULL;
 
     /* Read the mechlist & other capabilities */
     while(1) {
@@ -254,6 +254,7 @@ int mupdate_connect(const char *server, const char *port,
     }
 
     free(mechlist);
+    mechlist = NULL;
 
     /* xxx unclear that this is correct, but it prevents a memory leak */
     if(local_cbs) free_callbacks(cbs);
