@@ -72,7 +72,7 @@
  * may contain an explanatory message.
  *
  *
- * $Id: smmapd.c,v 1.10 2004/09/09 16:21:28 shadow Exp $
+ * $Id: smmapd.c,v 1.13 2005/04/13 15:42:04 shadow Exp $
  */
 
 #include <config.h>
@@ -157,7 +157,6 @@ int service_init(int argc, char **argv, char **envp)
     setproctitle_init(argc, argv, envp);
 
     signals_set_shutdown(&shut_down);
-    signals_add_handlers();
     signal(SIGPIPE, SIG_IGN);
 
     BB = config_getstring(IMAPOPT_POSTUSER);
@@ -200,6 +199,7 @@ int service_main(int argc __attribute__((unused)),
     r = begin_handling();
 
     shut_down(r);
+    return 0;
 }
 
 int verify_user(const char *key, long quotacheck,
@@ -265,6 +265,14 @@ int verify_user(const char *key, long quotacheck,
 	long aclcheck = !user ? ACL_POST : 0;
 	int type;
 	char *acl;
+        char *path;
+        char *c;
+        struct hostent *hp;
+        char *host;
+        struct sockaddr_in sin,sfrom;
+        char buf[512];
+        int soc, x, rc;
+
 	/*
 	 * check to see if mailbox exists and we can append to it:
 	 *
@@ -272,10 +280,10 @@ int verify_user(const char *key, long quotacheck,
 	 * - don't care about ACL on INBOX (always allow post)
 	 * - don't care about message size (1 msg over quota allowed)
 	 */
-	r = mboxlist_detail(namebuf, &type, NULL, NULL, &acl, NULL);
+	r = mboxlist_detail(namebuf, &type, &path, NULL, &acl, NULL);
 	if (r == IMAP_MAILBOX_NONEXISTENT && config_mupdate_server) {
 	    kick_mupdate();
-	    r = mboxlist_detail(namebuf, &type, NULL, NULL, &acl, NULL);
+	    r = mboxlist_detail(namebuf, &type, &path, NULL, &acl, NULL);
 	}
 
 	if (!r && (type & MBTYPE_REMOTE)) {
@@ -288,9 +296,55 @@ int verify_user(const char *key, long quotacheck,
 	    if ((access & aclcheck) != aclcheck) {
 		r = (access & ACL_LOOKUP) ?
 		    IMAP_PERMISSION_DENIED : IMAP_MAILBOX_NONEXISTENT;
+                return r;
 	    } else {
-		r = 0;
-	    }
+                r = 0;
+            }
+
+            /* proxy the request to the real backend server to 
+             * check the quota.  if this fails, just return 0
+             * (asuume under quota)
+             */
+
+            host = strdup(path);
+            c = strchr(host, '!');
+            if (c) *c = 0;
+
+            syslog(LOG_ERR, "verify_user(%s) proxying to host %s",
+                   namebuf, host);
+
+            hp = gethostbyname(host);
+            if (hp == (struct hostent*) 0) {
+               syslog(LOG_ERR, "verify_user(%s) failed: can't find host %s",
+                      namebuf, host);
+               return r;
+            }
+
+            soc = socket(PF_INET, SOCK_STREAM, 0);
+            memcpy(&sin.sin_addr.s_addr,hp->h_addr,hp->h_length);
+            sin.sin_family = AF_INET;
+
+            /* XXX port should be configurable */
+            sin.sin_port = htons(12345);
+
+            if (connect(soc,(struct sockaddr *) &sin, sizeof(sin)) < 0) { 
+                syslog(LOG_ERR, "verify_user(%s) failed: can't connect to %s", 
+                       namebuf, host);
+               return r;
+            }
+
+            sprintf(buf,"%d:cyrus %s,%c",strlen(key)+6,key,4);
+            sendto(soc,buf,strlen(buf),0,(struct sockaddr *)&sin,sizeof(sin));
+
+            x = sizeof(sfrom);
+            rc = recvfrom(soc,buf,512,0,(struct sockaddr *)&sfrom,&x);
+ 
+            buf[rc] = '\0';
+            close(soc);
+
+	    prot_printf(map_out, "%s", buf);
+            return -1;   /* tell calling function we already replied */
+
 	} else if (!r) {
 	    r = append_check(namebuf, MAILBOX_FORMAT_NORMAL, authstate,
 			     aclcheck, quotacheck > 0 ? 0 : quotacheck);
@@ -316,7 +370,10 @@ int begin_handling(void)
 	char *mapname = NULL, *key = NULL;
 	const char *errstring = NULL;
 
-	signals_poll();
+	if (signals_poll() == SIGHUP) {
+	    /* caught a SIGHUP, return */
+	    return 0;
+	}
 
 	while (isdigit(c)) {
 	    sawdigit = 1;
@@ -360,6 +417,10 @@ int begin_handling(void)
 	}
 
 	switch (r) {
+        case -1:
+            /* reply already sent */
+            break;
+
 	case 0:
 	    prot_printf(map_out, "%d:OK %s,", 3+strlen(key), key);
 	    break;
