@@ -1,7 +1,7 @@
 /* imtest.c -- IMAP/POP3/NNTP/LMTP/SMTP/MUPDATE/MANAGESIEVE test client
  * Ken Murchison (multi-protocol implementation)
  * Tim Martin (SASL implementation)
- * $Id: imtest.c,v 1.103 2004/08/04 13:03:18 ken3 Exp $
+ * $Id: imtest.c,v 1.107 2006/01/20 20:31:23 jeaton Exp $
  *
  * Copyright (c) 1998-2003 Carnegie Mellon University.  All rights reserved.
  *
@@ -328,8 +328,7 @@ int	tls_cipher_algbits = 0;
 static int set_cert_stuff(SSL_CTX * ctx, char *cert_file, char *key_file)
 {
     if (cert_file != NULL) {
-	if (SSL_CTX_use_certificate_file(ctx, cert_file,
-					 SSL_FILETYPE_PEM) <= 0) {
+	if (SSL_CTX_use_certificate_chain_file(ctx, cert_file) <= 0) {
 	    printf("unable to get certificate from '%s'\n", cert_file);
 	    return (0);
 	}
@@ -1086,12 +1085,12 @@ int auth_sasl(struct sasl_cmd_t *sasl_cmd, char *mechlist)
 	    prot_write(pout, inbase64, inbase64len);
 
 	    out = NULL;
-	} else if(sendliteral) {
+	} else if (sendliteral) {
 	    /* If we had no response, we still need to send the
 	       empty literal in this case */
 	    printf("{0+}\r\nC: ");
 	    prot_printf(pout, "{0+}\r\n");
-	} else {
+	} else if (!initial_response) {
 	    printf("C: ");
 	}
       noinitresp:
@@ -1283,7 +1282,10 @@ static void interactive(struct protocol_t *protocol, char *filename)
     FD_SET(fd_out, &write_set);
     FD_SET(sock, &write_set);
     
-    nfds = getdtablesize();
+    nfds = fd;
+    if (nfds < sock) nfds = sock;
+    if (nfds < fd_out) nfds = fd_out;
+    nfds++;
     
     if (filename != NULL) {
 	donewritingfile = 0;
@@ -1969,41 +1971,6 @@ static int pop3_do_auth(struct sasl_cmd_t *sasl_cmd, void *rock,
 
 /********************************** NNTP *************************************/
 
-/*
- * Parse a mech list of the form: ... SASL:foo,bar ...
- *
- * Return: string with mechs separated by spaces
- *
- */
-
-static char *nntp_parse_mechlist(const char *str,
-				 struct protocol_t *prot __attribute__((unused)))
-{
-    char *ret;
-    char *tmp;
-    int num = 0;
-    
-    tmp = strstr(str, " SASL:") + 6;
-    if (isspace((int) *tmp)) return NULL;
-
-    ret = xzmalloc(strlen(tmp)+1);
-    do {
-	char *end = tmp;
-	
-	while ((*end != ',') && (*end != ' ') && (*end != '\0')) end++;
-	
-	/* add entry to list */
-	if (num++ > 0) strcat(ret, " ");
-	strlcat(ret, tmp, strlen(ret) + (end - tmp) + 1);
-
-	/* reset the string */
-	tmp = end;
-
-    } while (*tmp++ != '\0');
-    
-    return ret;
-}
-
 static int auth_nntp(void)
 {
     char str[1024];
@@ -2242,7 +2209,7 @@ static struct protocol_t protocols[] = {
     },
     { "nntp", "nntps", "nntp",
       { 0, "20", NULL },
-      { "LIST EXTENSIONS", ".", "STARTTLS", " SASL:", &nntp_parse_mechlist },
+      { "CAPABILITIES", ".", "STARTTLS", "SASL ", NULL },
       { "STARTTLS", "382", "580", 0 },
       { "AUTHINFO SASL", 512, 0, "28", "48", "383 ", "*", &nntp_parse_success },
       &nntp_do_auth, { "QUIT", "205" }, NULL, NULL, NULL
