@@ -41,7 +41,7 @@
  *
  */
 /*
- * $Id: prot.c,v 1.80 2003/06/04 03:27:54 rjs3 Exp $
+ * $Id: prot.c,v 1.86 2004/02/27 22:08:56 rjs3 Exp $
  */
 
 #include <config.h>
@@ -59,15 +59,27 @@
 #include <unistd.h>
 #endif
 #include <sys/types.h>
-#include <sys/time.h>
+#include <sys/stat.h>
 #include <netinet/in.h>
 #ifdef HAVE_SYS_SELECT_H
 #include <sys/select.h>
 #endif
 
-#include "prot.h"
-#include "xmalloc.h"
 #include "assert.h"
+#include "exitcodes.h"
+#include "map.h"
+#include "nonblock.h"
+#include "prot.h"
+#include "util.h"
+#include "xmalloc.h"
+
+/* Transparant protgroup structure */
+struct protgroup
+{
+    size_t nalloced; /* Number of nodes in the group */
+    size_t next_element; /* Node number of next group member */
+    struct protstream **group;
+};
 
 /*
  * Create a new protection stream for file descriptor 'fd'.  Stream
@@ -79,30 +91,18 @@ int write;
 {
     struct protstream *newstream;
 
-    newstream = (struct protstream *) xmalloc(sizeof(struct protstream));
+    newstream = (struct protstream *) xzmalloc(sizeof(struct protstream));
     newstream->buf = (unsigned char *) 
 	xmalloc(sizeof(char) * (PROT_BUFSIZE));
     newstream->buf_size = PROT_BUFSIZE;
     newstream->ptr = newstream->buf;
-    newstream->cnt = write ? PROT_BUFSIZE : 0;
     newstream->maxplain = PROT_BUFSIZE;
     newstream->fd = fd;
     newstream->write = write;
-    newstream->logfd = -1;
-    newstream->error = 0;
-    newstream->eof = 0;
-    newstream->read_timeout = 0;
-    newstream->dontblock = 0;
-    newstream->flushonread = 0;
-    newstream->readcallback_proc = 0;
-    newstream->readcallback_rock = 0;
-    newstream->waitevent = 0;
-    newstream->conn = NULL;
-    newstream->saslssf=0;
-
-#ifdef HAVE_SSL
-    newstream->tls_conn=NULL;
-#endif /* HAVE_SSL */
+    newstream->logfd = PROT_NO_FD;
+    newstream->big_buffer = PROT_NO_FD;
+    if(write)
+	newstream->cnt = PROT_BUFSIZE;
 
     return newstream;
 }
@@ -114,7 +114,14 @@ int prot_free(struct protstream *s)
 {
     if (s->error) free(s->error);
     free(s->buf);
+
+    if(s->big_buffer != PROT_NO_FD) {
+	map_free(&(s->bigbuf_base), &(s->bigbuf_siz));
+	close(s->big_buffer);
+    }
+
     free((char*)s);
+
     return 0;
 }
 
@@ -137,6 +144,11 @@ int prot_settls(struct protstream *s, SSL *tlsconn)
 {
     s->tls_conn = tlsconn;
 
+    /* Make nonblocking stuff to work similar to write() */
+    SSL_set_mode(tlsconn,
+		 SSL_MODE_ENABLE_PARTIAL_WRITE
+		 | SSL_MODE_ACCEPT_MOVING_WRITE_BUFFER);
+
     return 0;
 }
 
@@ -155,7 +167,8 @@ sasl_conn_t *conn;
 
     if (s->write && s->ptr != s->buf) {
 	/* flush any pending output */
-	prot_flush(s);
+	if(prot_flush_internal(s,0) == EOF)
+	    return EOF;
     }
    
     s->conn = conn;
@@ -168,8 +181,8 @@ sasl_conn_t *conn;
 
     if (s->write) {
 	int result;
-	const int *maxp;
-	int max;
+	const unsigned int *maxp;
+	unsigned int max;
 
 	/* ask SASL for layer max */
 	result = sasl_getprop(conn, SASL_MAXOUTBUF, (const void **) &maxp);
@@ -299,7 +312,10 @@ void prot_removewaitevent(struct protstream *s, struct prot_waitevent *event)
  */
 const char *prot_error(struct protstream *s)
 {
-    return s->error;
+    if(!s) return "bad protstream passed to prot_error";
+    else if(s->error) return s->error;
+    else if(s->eof) return PROT_EOF_STRING;
+    else return NULL;
 }
 
 /*
@@ -369,7 +385,10 @@ int prot_fill(struct protstream *s)
 		    s->readcallback_proc = 0;
 		    s->readcallback_rock = 0;
 		}
-		if (s->flushonread) prot_flush(s->flushonread);
+		/* Request a flush of the buffer.  If we are a blocking
+		   read stream, force the flush */
+		if (s->flushonread)
+		    prot_flush_internal(s->flushonread, !s->dontblock);
 	    }
 	    else {
 		haveinput = 1;
@@ -516,27 +535,21 @@ int prot_fill(struct protstream *s)
 /*
  * Write out any buffered data in the stream 's'
  */
-int prot_flush(struct protstream *s)
+int prot_flush(struct protstream *s) 
 {
-    unsigned char *ptr = s->buf;
-    int left = s->ptr - s->buf;
-    int n;
-    const char *encoded_output;
+    return prot_flush_internal(s, 1);
+}
 
-    assert(s->write);
-    assert(s->cnt >= 0);
-
-    if (s->eof || s->error) {
-	s->ptr = s->buf;
-	s->cnt = 1;
-	return EOF;
-    }
-    if (!left) return 0;
-
-    if (s->logfd != -1) {
+/* Do the logging part of prot_flush */
+static void prot_flush_log(struct protstream *s) 
+{
+    if(s->logfd != PROT_NO_FD) {
+	unsigned char *ptr = s->buf;
+	int left = s->ptr - s->buf;
+	int n;
 	time_t newtime;
 	char timebuf[20];
-
+	
 	time(&newtime);
 	snprintf(timebuf, sizeof(timebuf), ">%ld>", newtime);
 	write(s->logfd, timebuf, strlen(timebuf));
@@ -551,17 +564,21 @@ int prot_flush(struct protstream *s)
 		left -= n;
 	    }
 	} while (left);
-	left = s->ptr - s->buf;
-	ptr = s->buf;
     }
+}
+
+/* Do the encoding part of prot_flush */
+static int prot_flush_encode(struct protstream *s,
+			     const char **output_buf,
+			     int *output_len) 
+{
+    unsigned char *ptr = s->buf;
+    int left = s->ptr - s->buf;
 
     if (s->saslssf != 0) {
 	/* encode the data */
-	unsigned int outlen;
-	int result;
-	
-	result = sasl_encode(s->conn, (char *) ptr, left, 
-			     &encoded_output, &outlen);
+	int result = sasl_encode(s->conn, (char *) ptr, left, 
+				 output_buf, output_len);
 	if (result != SASL_OK) {
 	    char errbuf[256];
 	    const char *ed = sasl_errdetail(s->conn);
@@ -570,41 +587,242 @@ int prot_flush(struct protstream *s)
 		     sasl_errstring(result, NULL, NULL),
 		     ed ? ed : "no detail");
 	    s->error = xstrdup(errbuf);
+	    
 	    return EOF;
 	}
-	
-	ptr = (unsigned char *) encoded_output;
-	left = outlen;
+    } else {
+	*output_buf = ptr;
+	*output_len = left;
     }
+    return 0;
+}
 
-    /* Write out the data */
+/* A wrapper for write() that handles SSL and EINTR */
+static int prot_flush_writebuffer(struct protstream *s,
+				  const char *buf, size_t len) 
+{
+    int n;
+    
     do {
 #ifdef HAVE_SSL
 	if (s->tls_conn != NULL) {
-	    n = SSL_write(s->tls_conn, (char *) ptr, left);
+	    n = SSL_write(s->tls_conn, (char *)buf, len);
 	} else {
-	    n = write(s->fd, ptr, left);
+	    n = write(s->fd, buf, len);
 	}
 #else  /* HAVE_SSL */
-	n = write(s->fd, ptr, left);
+	n = write(s->fd, buf, len);
 #endif /* HAVE_SSL */
-	if (n == -1 && errno != EINTR) {
-	    s->error = xstrdup(strerror(errno));
-            /* Reset the output buffer, we are returning EOF */
-            s->ptr = s->buf;
-            s->cnt = s->maxplain;
-	    return EOF;
+    } while (n == -1 && errno == EINTR);
+
+    return n;
+}
+
+int prot_flush_internal(struct protstream *s, int force)
+{
+    int n;
+    int save_dontblock = s->dontblock;
+
+    const char *ptr = s->buf; /* Memory buffer info */
+    int left = s->ptr - s->buf;
+
+    assert(s->write);
+    assert(s->cnt >= 0);
+
+    /* Is this protstream finished? */
+    if (s->eof || s->error) {
+	s->ptr = s->buf;
+	s->cnt = 1;
+	return EOF;
+    }
+
+    /* make sure that the main file descriptor is set up to
+     * be blocking or nonblocking based on the configuration of the
+     * protstream and the force flag */
+    if(force)
+	s->dontblock = 0;
+    
+    if(s->dontblock != s->dontblock_isset) {
+	nonblock(s->fd,s->dontblock);
+	s->dontblock_isset = s->dontblock;
+    }
+    
+    /* end protstream setup */
+
+    /* If we're doing a blocking write, flush the buffers, bigbuffer first */
+    if(!s->dontblock) {
+	if(s->big_buffer != PROT_NO_FD) {
+	    /* Write the bigbuffer */
+	    do {
+		n = prot_flush_writebuffer(s, s->bigbuf_base + s->bigbuf_pos,
+					   s->bigbuf_len - s->bigbuf_pos);
+		if(n == -1) {
+		    s->error = xstrdup(strerror(errno));
+		    goto done;
+		} else if (n > 0) {
+		    s->bigbuf_pos += n;
+		}
+	    } while(s->bigbuf_len != s->bigbuf_pos);
+
+	    /* Free the bigbuffer */
+	    map_free(&(s->bigbuf_base), &(s->bigbuf_siz));
+	    close(s->big_buffer);
+	    s->bigbuf_len = s->bigbuf_pos = 0;
+	    s->big_buffer = PROT_NO_FD;
 	}
 
-	if (n > 0) {
-	    ptr += n;
-	    left -= n;
+	/* Is there anything in the memory buffer? */
+	if(!left) {
+	    goto done;
 	}
-    } while (left);
 
-    /* Reset the output buffer */
+	/* Do a regular write of whatever is left */
+
+	/* Log and Encode it */
+	prot_flush_log(s);
+
+	if(prot_flush_encode(s, &ptr, &left) == EOF) {
+	    /* s->error set by prot_flush_encode */
+	    goto done;
+	}
+
+	/* Write it to descriptor */
+	do {
+	    n = prot_flush_writebuffer(s, ptr, left);
+	    if(n == -1) {
+		s->error = xstrdup(strerror(errno));
+		goto done;
+	    } else if (n > 0) {
+		ptr += n;
+		left -= n;
+	    }
+	} while(left);
+    } else { /* Nonblocking */
+	/* If we've been feeding a bigbuffer, write out from the current
+	 * position as much as we can */
+	if (s->big_buffer != PROT_NO_FD) {
+	    /* Write what we can. */
+	    n = prot_flush_writebuffer(s, s->bigbuf_base + s->bigbuf_pos,
+				       s->bigbuf_len - s->bigbuf_pos);
+
+	    if(n == -1 && errno == EAGAIN) {
+		/* No room in the pipe, but we don't care */
+		n = 0;
+	    } else if(n == -1) {
+		s->error = xstrdup(strerror(errno));
+		goto done;
+	    }
+
+	    if (n > 0) {
+		s->bigbuf_pos += n;
+	    }
+	}
+
+	/* If there isn't anything in the memory buffer, we're done now */
+	if(!left) {
+	    goto done;
+	}
+
+	/* Prepare the data in the memory buffer */
+	prot_flush_log(s);
+	
+	/* Encode it */
+	if(prot_flush_encode(s, &ptr, &left) == EOF) {
+	    /* prot_flush_encode set s->error */
+	    goto done;
+	}
+
+	if(s->big_buffer == PROT_NO_FD || s->bigbuf_pos == s->bigbuf_len) {
+	    /* No bigbuffer currently open (or we've written the current
+	       one to its entirety), so write what we can from memory */
+
+	    n = prot_flush_writebuffer(s, ptr, left);
+
+	    if(n == -1 && errno == EAGAIN) {
+		/* No room in the pipe, but we don't care */
+		n = 0;
+	    } else if(n == -1) {
+		s->error = xstrdup(strerror(errno));
+		goto done;
+	    }
+
+	    if(n > 0) {
+		ptr += n;
+		left -= n;
+	    }
+	}
+
+	/* if there is data still to send, it needs to go to the bigbuffer */
+	if(left) {
+	    struct stat sbuf;
+	    
+	    if(s->big_buffer == PROT_NO_FD) {
+		/* open new bigbuffer */
+		int fd = create_tempfile();
+		if(fd == -1) {
+		    s->error = xstrdup(strerror(errno));
+		    goto done;
+		}
+
+		s->big_buffer = fd;
+	    }
+
+	    do {
+		n = write(s->big_buffer, ptr, left);
+		if (n == -1 && errno != EINTR) {
+		    syslog(LOG_ERR, "write to protstream buffer failed: %s",
+			   strerror(errno));
+		    
+		    fatal("write to big buffer failed", EC_OSFILE);
+		}
+		if (n > 0) {
+		    ptr += n;
+		    left -= n;
+		}
+	    } while (left);
+
+	    /* We did a write to the bigbuffer, refresh the memory map */
+	    if (fstat(s->big_buffer, &sbuf) == -1) {
+		syslog(LOG_ERR, "IOERROR: fstating temp protlayer buffer: %m");
+		fatal("failed to fstat protlayer buffer", EC_IOERR);
+	    }
+	    
+	    s->bigbuf_len = sbuf.st_size;
+
+	    map_refresh(s->big_buffer, 0, &(s->bigbuf_base), &(s->bigbuf_siz),
+			s->bigbuf_len, "temp protlayer buffer", NULL);
+	}
+	
+    } /* end of blocking/nonblocking if statment */
+
+    /* Reset the memory buffer -- should be done on EOF or on success. */
     s->ptr = s->buf;
     s->cnt = s->maxplain;
+        
+ done:
+    /* are we done with the big buffer? If so, free it. This includes
+     * when we exit with error */
+    if(s->big_buffer != PROT_NO_FD &&
+       (s->bigbuf_pos == s->bigbuf_len || s->error)) {
+	map_free(&(s->bigbuf_base), &(s->bigbuf_siz));
+	close(s->big_buffer);
+	s->bigbuf_len = s->bigbuf_pos = 0;
+	s->big_buffer = PROT_NO_FD;
+    }
+
+    if(force) {
+	/* we don't need to call nonblock() again, because it will be
+	 * set correctly on the next prot_flush_internal() anyway */
+	s->dontblock = save_dontblock;
+    }
+    
+    /* If we are exiting with an error, we should clear our memory buffer 
+     * and set our return code */
+    if(s->error) {
+        s->ptr = s->buf;
+        s->cnt = s->maxplain;
+	return EOF;
+    }
 
     return 0;
 }
@@ -626,7 +844,7 @@ int prot_write(struct protstream *s, const char *buf, unsigned len)
 	buf += s->cnt;
 	len -= s->cnt;
 	s->cnt = 0;
-	if (prot_flush(s) == EOF) return EOF;
+	if (prot_flush_internal(s,0) == EOF) return EOF;
     }
     memcpy(s->ptr, buf, len);
     s->ptr += len;
@@ -745,6 +963,171 @@ int prot_read(struct protstream *s, char *buf, unsigned size)
     return size+1;
 }
 
+/*
+ * select() for protection streams, read only
+ * Also supports selecting on an extra file descriptor
+ *
+ * returns # of protstreams with pending data (including the extra fd)
+ *
+ * Only works for readable protstreams
+ */ 
+int prot_select(struct protgroup *readstreams, int extra_read_fd,
+		struct protgroup **out, int *extra_read_flag,
+		struct timeval *timeout) 
+{
+    struct protstream *s, *timeout_prot = NULL;
+    struct protgroup *retval = NULL;
+    int max_fd, found_fds = 0;
+    int i;
+    fd_set rfds;
+    int have_readtimeout = 0;
+    struct timeval my_timeout;
+    struct prot_waitevent *event;
+    time_t now = time(NULL);
+    time_t read_timeout = 0;
+    
+    assert(readstreams || extra_read_fd != PROT_NO_FD);
+    assert(extra_read_fd == PROT_NO_FD || extra_read_flag);
+    assert(out);
+
+    /* Initialize things we might use */
+    errno = 0;
+    found_fds = 0;
+    FD_ZERO(&rfds);
+
+    /* If extra_read_fd is PROT_NO_FD, then the first protstream
+     * will override it */
+    max_fd = extra_read_fd;
+
+    for(i = 0; i<readstreams->next_element; i++) {
+	int have_thistimeout = 0; /* used to compute the minimal timeout for */
+	time_t this_timeout = 0;   /* this stream */
+	
+	s = readstreams->group[i];
+
+	assert(!s->write);
+
+	have_thistimeout = 0;
+	
+	/* scan for waitevent callbacks */
+	for (event = s->waitevent; event; event = event->next)
+	{
+	    if(!have_thistimeout || event->mark - now < this_timeout) {
+		this_timeout = event->mark - now;
+		have_thistimeout = 1;
+	    }
+	}
+	
+	/* check the idle timeout on this one as well */
+	if(!have_thistimeout || this_timeout > s->read_timeout)
+	    this_timeout = s->read_timeout;
+
+	if(!have_readtimeout && !s->dontblock) {
+	    read_timeout = now + this_timeout;
+	    have_readtimeout = 1;
+	    if(!timeout || read_timeout <= timeout->tv_sec)
+		timeout_prot = s;
+	} else if(!s->dontblock) {
+	    time_t new_timeout;
+	    new_timeout = now + this_timeout;
+	    if(new_timeout < read_timeout) {
+		read_timeout = new_timeout;
+		if(!timeout || read_timeout <= timeout->tv_sec)
+		    timeout_prot = s;
+	    }
+	}
+	    
+	FD_SET(s->fd, &rfds);
+	if(s->fd > max_fd)
+	    max_fd = s->fd;
+
+	/* Is something currently pending in our protstream's buffer? */
+	if(s->cnt > 0) {
+	    found_fds++;
+
+	    if(!retval)
+		retval = protgroup_new(readstreams->next_element + 1);
+
+	    protgroup_insert(retval, s);
+	    
+	}
+#ifdef HAVE_SSL
+	else if(s->tls_conn != NULL && SSL_pending(s->tls_conn)) {
+	    found_fds++;
+
+	    if(!retval)
+		retval = protgroup_new(readstreams->next_element + 1);
+
+	    protgroup_insert(retval, s);
+	}
+#endif
+    }
+
+    /* xxx we should probably do a nonblocking select on the remaining
+     * protstreams instead of skipping this part entirely */
+    if(!retval) {
+	time_t sleepfor;
+
+	/* do a select */
+	if(extra_read_fd != PROT_NO_FD) {
+	    /* max_fd started with atleast extra_read_fd */
+	    FD_SET(extra_read_fd, &rfds);
+	}
+
+	if(read_timeout < now)
+	    sleepfor = 0;
+	else
+	    sleepfor = read_timeout - now;
+
+	/* If we don't have a timeout structure, and we need one, use
+	 * a local version.  Otherwise, make sure that we are timing out
+	 * for the right reason */
+	if((!timeout && have_readtimeout)
+	   || (timeout && read_timeout < timeout->tv_sec)) {
+	    if(!timeout)
+		timeout = &my_timeout;
+	    timeout->tv_sec = sleepfor;
+	    timeout->tv_usec = 0;
+	}
+
+	if(select(max_fd + 1, &rfds, NULL, NULL, timeout) == -1)
+	    return -1;
+
+	/* Reset now */
+	now = time(NULL);
+
+	if(extra_read_fd != PROT_NO_FD && FD_ISSET(extra_read_fd, &rfds)) {
+	    *extra_read_flag = 1;
+	    found_fds++;
+	} else if(extra_read_flag) {
+	    *extra_read_flag = 0;
+	}
+	
+	for(i = 0; i<readstreams->next_element; i++) {
+	    s = readstreams->group[i];
+
+	    if(FD_ISSET(s->fd, &rfds)) {
+		found_fds++;
+
+		if(!retval)
+		    retval = protgroup_new(readstreams->next_element + 1);
+
+		protgroup_insert(retval, s);
+	    } else if(s == timeout_prot && now >= read_timeout) {
+		/* If we timed out, be sure to add the protstream we were
+		 * waiting for, even if it didn't show up */
+		if(!retval)
+		    retval = protgroup_new(readstreams->next_element + 1);
+
+		protgroup_insert(retval, s);
+	    }
+	}	
+    }
+    
+    *out = retval;
+    return found_fds;
+}
+
 /*
  * Version of fgets() that works with protection streams.
  */
@@ -755,7 +1138,7 @@ char *prot_fgets(char *buf, unsigned size, struct protstream *s)
 
     assert(!s->write);
 
-    if (size < 2) return 0;
+    if (size < 2 || s->eof) return 0;
     size--;
 
     while (size && (c = prot_getc(s)) != EOF) {
@@ -768,6 +1151,73 @@ char *prot_fgets(char *buf, unsigned size, struct protstream *s)
     return buf;
 }
 
+/* Handle protgroups */
+/* Create a new protgroup of the given size, or 32 if size is 0 */
+struct protgroup *protgroup_new(size_t size) 
+{
+    struct protgroup *ret = xmalloc(sizeof(struct protgroup));
+
+    if(!size) size = PROTGROUP_SIZE_DEFAULT;
+
+    ret->nalloced = size;
+    ret->next_element = 0;
+    ret->group = xzmalloc(size * sizeof(struct protstream *));
+
+    return ret;
+}
+
+struct protgroup *protgroup_copy(struct protgroup *src)
+{
+    struct protgroup *dest;
+    assert(src);
+    dest = protgroup_new(src->nalloced);
+    if(src->next_element) {
+	memcpy(dest->group, src->group,
+	       src->next_element * sizeof(struct protstream *));
+    }
+    return dest;
+}
+
+void protgroup_reset(struct protgroup *group) 
+{
+    if(group) {
+	memset(group->group, 0,
+	       group->next_element * sizeof(struct protstream *));
+	group->next_element = 0;
+    }
+}
+
+void protgroup_free(struct protgroup *group) 
+{
+    if(group) {
+	assert(group->group);
+	free(group->group);
+	free(group);
+    }
+}
+
+void protgroup_insert(struct protgroup *group, struct protstream *item) 
+{
+    assert(group);
+    assert(item);
+    /* Double size of the protgroup if we're at our limit */
+    if(group->next_element == group->nalloced) {
+	group->nalloced *= 2;
+	group->group = xrealloc(group->group,
+				group->nalloced * sizeof(struct protstream *));
+    }
+    /* Insert the item on the end of the group */
+    group->group[group->next_element++] = item;
+}
+
+struct protstream *protgroup_getelement(struct protgroup *group,
+					size_t element) 
+{
+    assert(group);
+    if(element >= group->next_element) return NULL;
+    else return group->group[element];
+}
+
 /* function versions of the macros */
 #undef prot_getc
 #undef prot_ungetc
@@ -801,10 +1251,8 @@ int prot_putc(int c, struct protstream *s)
 
     *s->ptr++ = c;
     if (--s->cnt == 0) {
-	return prot_flush(s);
+	return prot_flush_internal(s,0);
     } else {
 	return 0;
     }
 }
-
-
